const topojson = require('topojson');

module.exports = class GeometryTree {
  constructor(topology, label) {
    this.topology = topology;
    this.geometries = [];
    this.merged;
    this.children;
    this.label = label;
  }

  add(geometry, child, ...restOfPath) {

    if (child) {
      if (!this.children) {
        this.children = {};
      }

      if (!this.children[child]) {
        this.children[child] = new GeometryTree(this.topology, child);
      }

      this.children[child].add(geometry, ...restOfPath);
    }

    this.geometries.push(geometry);
  }

  childGeometriesMerged() {
    const geometries = [];
    for (let a in this.children) {
      geometries.push(this.children[a].mergeArcs());
    }
    return geometries;
  }

  mergeArcs() {
    if (!this.merged) {
      if (this.children) {
        this.merged = topojson.mergeArcs(this.topology, this.childGeometriesMerged());
      }
      else {
        this.merged = topojson.mergeArcs(this.topology, this.geometries);
      }
    }

    return this.merged;
  }

  merge() {
    const feature = topojson.feature(this.topology, this.mergeArcs());

    feature.properties = {
      label: this.label
    };

    return feature;
  }

  mergeToArray() {

    const arr = [];

    for (let a in this.children) {
      arr.push(this.children[a].merge());
    }

    return arr;
  }

  mergeAllToArray() {
    const geometryCollection = {
      type: 'GeometryCollection',
      geometries: this.geometries
    };

    return topojson.feature(this.topology, geometryCollection).features;
  }

  mesh(border) {
    const geometryCollection = {
      type: 'GeometryCollection',
      geometries: this.childGeometriesMerged()
    };

    if (border) {
      return topojson.mesh(this.topology, geometryCollection);
    }
    else {
      return topojson.mesh(this.topology, geometryCollection, (a, b) => a !== b);
    }
  }

  meshAll(border) {
    const geometryCollection = {
      type: 'GeometryCollection',
      geometries: this.geometries
    };

    if (border) {
      return topojson.mesh(this.topology, geometryCollection);
    }
    else {
      return topojson.mesh(this.topology, geometryCollection, (a, b) => a !== b);
    }
  }

  getDescendent(child, ...restOfPath) {
    if (!child) {
      return this;
    }

    if (!this.children || !this.children[child]) {
      return false;
    }

    return this.children[child].getDescendent(...restOfPath);
  }
};