var webpack = require('webpack');
var config = {
  entry: './src/Widget',
  output: {
    path: __dirname + '/dist',
    publicPath: '/dist/',
    filename: 'bundle.js',
    libraryTarget: 'umd',
    library: 'CanadaFsaMap'
  },
  plugins: [
    new webpack.BannerPlugin('Copyright 2016 Domo Inc.')
  ],
  module: {
    loaders: [
      {
        test: /\.js$/,
        exclude: /(node_modules|bower_components)/,
        loader: 'babel',
        query: {
          presets: ['es2015']
        }
      },
      { test: /\.css$/, loader: "style-loader!css-loader" }
    ]
  },
  externals: {
    d3: 'd3',
    'd3.chart': 'd3.chart',
    'topojson': 'topojson',
    '@domoinc/base-widget': {
      root: 'BaseWidget',
      commonjs: '@domoinc/base-widget',
      commonjs2: '@domoinc/base-widget',
      amd: 'base-widget'
    },
    '@domoinc/summary-number': {
      root: 'SummaryNumber',
      commonjs: '@domoinc/summary-number',
      commonjs2: '@domoinc/summary-number',
      amd: 'summary-number'
    },
    '@domoinc/da-theme2': {
      root: ['da','theme2'],
      commonjs: '@domoinc/da-theme2',
      commonjs2: '@domoinc/da-theme2',
      amd: 'theme2'
    },
    '@domoinc/domo-tooltip': {
      root: 'DomoTooltip',
      commonjs: '@domoinc/domo-tooltip',
      commonjs2: '@domoinc/domo-tooltip',
      amd: 'domo-tooltip'
    }
  }
};

config.setDev = function() {
  var externals = config.externals;
  delete config.externals;
  config.entry = {
    app: config.entry,
    vendor: Object.keys(externals)
  };
  config.plugins.push(new webpack.optimize.CommonsChunkPlugin(/* chunkName= */"vendor", /* filename= */"vendor.bundle.js"));
};

if (process.env.NODE_ENV === 'development') {
  config.setDev();
}

module.exports = config;
